@@ -1,21 +1,26 @@
-# -*- coding: utf-8 -*-
-# Copyright © 2025 Spyder Project Contributors
-# Licensed under the terms of the MIT License
-# (see spyder/__init__.py for details)
+# -----------------------------------------------------------------------------
+# Copyright (c) 2024- Spyder Project Contributors
+#
+# Released under the terms of the MIT License
+# (see LICENSE.txt in the project root directory for details)
+# -----------------------------------------------------------------------------
 
 """
-Spyder AsyncDispatcher API.
+Class decorator and helpers for running code in async loops within Spyder.
 
-This module provides an class decorator `AsyncDispatcher` to run coroutines on
-dedicated async loops, including utilities for patching loops, managing
-concurrency tasks, and executing callbacks safely within Qt applications.
+This module provides a class decorator :class:`AsyncDispatcher` to run
+coroutines in dedicated async loops, as well as including utilities for
+patching loops, managing concurrency tasks, and executing callbacks safely
+within Qt applications.
 """
 
 from __future__ import annotations
 
+# Standard library imports
 import asyncio
 import asyncio.events
 import atexit
+import collections.abc
 import contextlib
 import functools
 import logging
@@ -29,23 +34,37 @@ from asyncio.futures import (
 from asyncio.tasks import (
     _current_tasks,  # type: ignore[attr-defined] # noqa: PLC2701
 )
-from concurrent.futures import CancelledError, Future
+from concurrent.futures import Future
 from heapq import heappop
 
-if sys.version_info >= (3, 10):
-    from typing import ParamSpec  # noqa: ICN003
-else:
+if sys.version_info < (3, 10):
     from typing_extensions import ParamSpec
+    from typing_extensions import TypeAlias
+else:
+    from typing import ParamSpec  # noqa: ICN003
+    from typing import TypeAlias  # noqa: ICN003
 
+# Third party imports
 from qtpy.QtCore import QCoreApplication, QEvent, QObject
 
+
 _logger = logging.getLogger(__name__)
 
-LoopID = typing.Union[typing.Hashable, asyncio.AbstractEventLoop]
+LoopID: TypeAlias = typing.Union[typing.Hashable, asyncio.AbstractEventLoop]
+"""Type alias, the union of a loop itself or the ID of one.
+
+Either an existing :class:`asyncio.AbstractEventLoop` or a
+:class:`~collections.abc.Hashable` to identify an existing or new one.
+"""
 
 _P = ParamSpec("_P")
-_T = typing.TypeVar("_T")
+"""The parameter types of a given async callable."""
+
 _RT = typing.TypeVar("_RT")
+"""The return type of a given async callable."""
+
+_T = typing.TypeVar("_T")
+"""An arbirary type parameter."""
 
 
 class AsyncDispatcher(typing.Generic[_RT]):
@@ -64,7 +83,7 @@ class AsyncDispatcher(typing.Generic[_RT]):
         loop: LoopID | None = ...,
         early_return: typing.Literal[True] = ...,
         return_awaitable: typing.Literal[False] = ...,
-    ): ...
+    ) -> None: ...
 
     @typing.overload
     def __init__(
@@ -73,7 +92,7 @@ class AsyncDispatcher(typing.Generic[_RT]):
         loop: LoopID | None = ...,
         early_return: typing.Literal[True] = ...,
         return_awaitable: typing.Literal[True] = ...,
-    ): ...
+    ) -> None: ...
 
     @typing.overload
     def __init__(
@@ -82,16 +101,16 @@ class AsyncDispatcher(typing.Generic[_RT]):
         loop: LoopID | None = ...,
         early_return: typing.Literal[False] = ...,
         return_awaitable: typing.Literal[False] = ...,
-    ): ...
+    ) -> None: ...
 
     @typing.overload
     def __init__(
-        self: AsyncDispatcher[typing.Awaitable[_T]],
+        self: AsyncDispatcher[collections.abc.Awaitable[_T]],
         *,
         loop: LoopID | None = ...,
         early_return: typing.Literal[False] = ...,
         return_awaitable: typing.Literal[True] = ...,
-    ): ...
+    ) -> None: ...
 
     def __init__(
         self,
@@ -99,61 +118,71 @@ class AsyncDispatcher(typing.Generic[_RT]):
         loop: LoopID | None = None,
         early_return: bool = True,
         return_awaitable: bool = False,
-    ):
+    ) -> None:
         """
         Decorate a coroutine to run in a specific event loop.
 
-        The `loop` parameter can be an existing loop or a hashable to identify
-        an existing/new one (to be) created by the AsyncDispatcher. If the
-        loop is not running, it will be started in a new thread and managed by
-        the AsyncDispatcher.
+        The ``loop`` parameter can be an existing
+        :class:`asyncio.AbstractEventLoop` or a
+        :class:`~collections.abc.Hashable` to identify an existing or new one
+        created by the :class:`!AsyncDispatcher`. If the loop is not running,
+        it will be started in a new thread and managed by the
+        :class:`!AsyncDispatcher`.
 
         This instance can be called with the same arguments as the coroutine it
-        wraps and will return a concurrent Future object, or an awaitable
-        Future for the current running event loop or the result of the
-        coroutine depending on the `early_return` and `return_awaitable`
-        parameters.
-
-        Usage
-        -----
-        Non-Blocking usage (returns a concurrent Future):
-        ```
-        @AsyncDispatcher()
-        async def my_coroutine(...):
-            ...
-
-        future = my_coroutine(...)  # Non-blocking call
-
-        result = future.result()  # Blocking call
-        ```
-
-        Blocking usage (returns the result):
-        ```
-        @AsyncDispatcher(early_return=False)
-        async def my_coroutine(...):
-            ...
-
-        result = my_coroutine(...)  # Blocking call
-        ```
-
-        Coroutine usage (returns an awaitable Future):
-        ```
-        @AsyncDispatcher(return_awaitable=True)
-        async def my_coroutine(...):
-            ...
-
-        result = await my_coroutine(...)  # Wait for the result to be ready
-        ```
+        wraps and will return a :class:`concurrent.futures.Future` object,
+        or an awaitable :class:`asyncio.Future` for the current running event
+        loop or the result of the coroutine depending on the ``early_return``
+        and ``return_awaitable`` parameters.
 
         Parameters
         ----------
-        loop : LoopID, optional (default: None)
-            The event loop to be used, by default get the current event loop.
-        early_return : bool, optional (default: True)
-            Return the coroutine as a concurrent Future before it is done.
-        return_awaitable : bool, optional (default: False)
-            Return the coroutine as an awaitable (asyncio) Future instead of a
-            concurrent Future. Idenpendently of the value of `early_return`.
+        loop : LoopID | None, optional
+            The event loop to be used, by default the current event loop.
+        early_return : bool, optional
+            Return the coroutine as a :class:`concurrent.futures.Future`
+            before it is done. ``True`` by default.
+        return_awaitable : bool, optional
+            Return the coroutine as an awaitable :class:`asyncio.Future`
+            instead of a :class:`concurrent.futures.Future`, independent of
+            the value of ``early_return``. ``False`` by default.
+
+
+        Examples
+        --------
+
+        Non-blocking usage (returns a :class:`concurrent.futures.Future`):
+
+        .. code-block:: python
+
+            @AsyncDispatcher()
+            async def my_coroutine(...):
+                ...
+
+            future = my_coroutine(...)  # Non-blocking call
+
+            result = future.result()  # Blocking call
+
+
+        Blocking usage (returns the result):
+
+        .. code-block:: python
+
+            @AsyncDispatcher(early_return=False)
+            async def my_coroutine(...):
+                ...
+
+            result = my_coroutine(...)  # Blocking call
+
+        Coroutine usage (returns an awaitable :class:`asyncio.Future`):
+
+        .. code-block:: python
+
+            @AsyncDispatcher(return_awaitable=True)
+            async def my_coroutine(...):
+                ...
+
+            result = await my_coroutine(...)  # Wait for the result to be ready
         """
         self._loop = self.get_event_loop(loop)
         self._early_return = early_return
@@ -161,28 +190,36 @@ class AsyncDispatcher(typing.Generic[_RT]):
 
     @typing.overload
     def __call__(
-        self: AsyncDispatcher[typing.Awaitable[_T]],
-        async_func: typing.Callable[_P, typing.Awaitable[_T]],
-    ) -> typing.Callable[_P, typing.Awaitable[_T]]: ...
+        self: AsyncDispatcher[collections.abc.Awaitable[_T]],
+        async_func: collections.abc.Callable[
+            _P, collections.abc.Awaitable[_T]
+        ],
+    ) -> collections.abc.Callable[_P, collections.abc.Awaitable[_T]]: ...
 
     @typing.overload
     def __call__(
         self: AsyncDispatcher[DispatcherFuture[_T]],
-        async_func: typing.Callable[_P, typing.Awaitable[_T]],
-    ) -> typing.Callable[_P, DispatcherFuture[_T]]: ...
+        async_func: collections.abc.Callable[
+            _P, collections.abc.Awaitable[_T]
+        ],
+    ) -> collections.abc.Callable[_P, DispatcherFuture[_T]]: ...
 
     @typing.overload
     def __call__(
         self: AsyncDispatcher[_T],
-        async_func: typing.Callable[_P, typing.Awaitable[_T]],
-    ) -> typing.Callable[_P, _T]: ...
+        async_func: collections.abc.Callable[
+            _P, collections.abc.Awaitable[_T]
+        ],
+    ) -> collections.abc.Callable[_P, _T]: ...
 
     def __call__(
         self,
-        async_func: typing.Callable[_P, typing.Awaitable[_T]],
-    ) -> typing.Callable[
+        async_func: collections.abc.Callable[
+            _P, collections.abc.Awaitable[_T]
+        ],
+    ) -> collections.abc.Callable[
         _P,
-        typing.Union[_T, DispatcherFuture[_T], typing.Awaitable[_T]],  # noqa: UP007
+        _T | DispatcherFuture[_T] | collections.abc.Awaitable[_T],
     ]:
         """
         Run the coroutine in the event loop.
@@ -196,12 +233,12 @@ class AsyncDispatcher(typing.Generic[_RT]):
 
         Returns
         -------
-        concurrent.Future or asyncio.Future or result of the coroutine
+        concurrent.futures.Future or asyncio.Future or _T
 
         Raises
         ------
         TypeError
-            If the function is not a coroutine function.
+            If ``async_func`` is not a coroutine function.
         """
         if not asyncio.iscoroutinefunction(async_func):
             msg = f"{async_func} is not a coroutine function"
@@ -209,15 +246,17 @@ class AsyncDispatcher(typing.Generic[_RT]):
 
         @functools.wraps(async_func)
         def wrapper(
-            *args: _P.args, **kwargs: _P.kwargs,
-        ) -> typing.Union[_T, DispatcherFuture[_T], typing.Awaitable[_T]]:  # noqa: UP007
+            *args: _P.args,
+            **kwargs: _P.kwargs,
+        ) -> _T | DispatcherFuture[_T] | collections.abc.Awaitable[_T]:
             task = run_coroutine_threadsafe(
                 async_func(*args, **kwargs),
                 loop=self._loop,
             )
             if self._return_awaitable:
                 return asyncio.wrap_future(
-                    task, loop=asyncio.get_running_loop(),
+                    task,
+                    loop=asyncio.get_running_loop(),
                 )
 
             if self._early_return:
@@ -231,9 +270,6 @@ class AsyncDispatcher(typing.Generic[_RT]):
     @staticmethod
     def _callback_task_done(future: Future):
         AsyncDispatcher._running_tasks.remove(future)
-        with contextlib.suppress(asyncio.CancelledError, CancelledError):
-            if (exception := future.exception()) is not None:
-                raise exception
 
     @classmethod
     def get_event_loop(
@@ -243,24 +279,24 @@ class AsyncDispatcher(typing.Generic[_RT]):
         """Get the event loop to run the coroutine.
 
         If the loop is not running, it will be started in a new thread and
-        managed by the AsyncDispatcher.
+        managed by the :class:`!AsyncDispatcher`.
 
         Parameters
         ----------
-        loop_id : LoopID, optional (default: None)
-            The event loop to be used, by default gets the current thread event
-            loop.
+        loop_id : LoopID | None, optional
+            The event loop to use, by default the current thread event loop.
 
-        Notes
-        -----
-        * If a hashable is provided, it will be used to identify the loop in
-          the AsyncDispatcher.
-        * If an event loop is provided, it will be used as the event loop in
-          the AsyncDispatcher.
+            .. note::
+
+                * If a :class:`~collections.abc.Hashable` is provided, it will
+                  be used to identify the loop in the
+                  :class:`!AsyncDispatcher`.
+                * If an event loop is provided, it will be used as the event
+                  loop in the :class:`!AsyncDispatcher`.
 
         Returns
         -------
-        AbstractEventLoop
+        asyncio.AbstractEventLoop
             The event loop to be used.
         """
         loop, loop_id = cls._fetch_event_loop(loop_id)
@@ -304,8 +340,10 @@ class AsyncDispatcher(typing.Generic[_RT]):
 
     @classmethod
     def __run_loop(
-        cls, loop_id: typing.Hashable, loop: asyncio.AbstractEventLoop,
-    ):
+        cls,
+        loop_id: typing.Hashable,
+        loop: asyncio.AbstractEventLoop,
+    ) -> None:
         if loop_id not in cls.__running_threads:
             with cls.__rlock:
                 if loop_id not in cls.__running_threads:
@@ -315,8 +353,13 @@ class AsyncDispatcher(typing.Generic[_RT]):
 
     @staticmethod
     @atexit.register
-    def close():
-        """Close the thread pool."""
+    def close() -> None:
+        """Close the thread pool.
+
+        Returns
+        -------
+        None
+        """
         if AsyncDispatcher.__closed:
             return
         AsyncDispatcher.cancel_all()
@@ -324,14 +367,30 @@ class AsyncDispatcher(typing.Generic[_RT]):
         AsyncDispatcher.__closed = True
 
     @classmethod
-    def cancel_all(cls):
-        """Cancel all running tasks."""
+    def cancel_all(cls) -> None:
+        """Cancel all running tasks.
+
+        Returns
+        -------
+        None
+        """
         for task in cls._running_tasks:
             task.cancel()
 
     @classmethod
-    def join(cls, timeout: float | None = None):
-        """Close all running loops and join the threads."""
+    def join(cls, timeout: float | None = None) -> None:
+        """Close all running loops and join the threads.
+
+        Parameters
+        ----------
+        timeout : float | None, optional
+            Seconds to wait for loops to exit and their threads to be joined.
+            By default, waits indefinitely.
+
+        Returns
+        -------
+        None
+        """
         for loop_id in list(cls.__running_threads.keys()):
             cls._stop_running_loop(loop_id, timeout)
 
@@ -345,11 +404,13 @@ class AsyncDispatcher(typing.Generic[_RT]):
         runner.join(timeout)
 
     @staticmethod
-    def QtSlot(func: typing.Callable[_P, None]) -> typing.Callable[_P, None]:  # noqa: N802
-        """Mark a function to be executed inside the main qt loop.
+    def QtSlot(
+        func: collections.abc.Callable[_P, None],
+    ) -> collections.abc.Callable[_P, None]:  # noqa: N802
+        """Mark a function to be executed inside the main Qt loop.
 
-        Set the `DispatcherFuture.QT_SLOT_ATTRIBUTE` attribute to the function
-        to mark it as a slot to be executed in the main Qt loop.
+        Sets the :attr:`DispatcherFuture.QT_SLOT_ATTRIBUTE` attribute on the
+        function to mark it as a slot to be executed in the main Qt loop.
 
         Parameters
         ----------
@@ -369,7 +430,9 @@ class _LoopRunner(threading.Thread):
     """A task runner that runs an asyncio event loop on a background thread."""
 
     def __init__(
-        self, loop_id: typing.Hashable, loop: asyncio.AbstractEventLoop,
+        self,
+        loop_id: typing.Hashable,
+        loop: asyncio.AbstractEventLoop,
     ):
         super().__init__(daemon=True, name=f"AsyncDispatcher-{loop_id}")
         self.__loop = loop
@@ -407,7 +470,7 @@ def _patch_loop_as_reentrant(loop):  # noqa: C901, PLR0915
 
     Parameters
     ----------
-    loop : AbstractEventLoop
+    loop : asyncio.AbstractEventLoop
         The event loop to be patched.
 
     Raises
@@ -454,9 +517,11 @@ def _patch_loop_as_reentrant(loop):  # noqa: C901, PLR0915
         timeout = (
             0
             if ready or self._stopping
-            else min(max(scheduled[0]._when - self.time(), 0), 86400)
-            if scheduled
-            else None
+            else (
+                min(max(scheduled[0]._when - self.time(), 0), 86400)
+                if scheduled
+                else None
+            )
         )
         event_list = self._selector.select(timeout)
         self._process_events(event_list)
@@ -545,7 +610,7 @@ def _patch_loop_as_reentrant(loop):  # noqa: C901, PLR0915
     cls._check_running = _check_running
     cls._num_runs_pending = 1 if loop.is_running() else 0
     cls._is_proactorloop = os.name == "nt" and issubclass(
-        cls, asyncio.ProactorEventLoop,
+        cls, asyncio.ProactorEventLoop
     )
     cls._nest_patched = True
 
@@ -553,7 +618,7 @@ def _patch_loop_as_reentrant(loop):  # noqa: C901, PLR0915
 class _QCallbackEvent(QEvent):
     """Event to execute a callback in the main Qt loop."""
 
-    def __init__(self, func: typing.Callable):
+    def __init__(self, func: collections.abc.Callable):
         super().__init__(QEvent.Type.User)
         self.func = func
 
@@ -568,21 +633,26 @@ class _QCallbackExecutor(QObject):
 class DispatcherFuture(Future, typing.Generic[_T]):
     """Represents the result of an asynchronous computation.
 
-    This class is a subclass of `concurrent.Future` that adds a `connect`
-    method to allow attaching callbacks to be executed in the main Qt loop.
+    This class is a subclass of :class:`concurrent.futures.Future` that adds
+    a :meth:`connect` method to allow attaching callbacks to be executed
+    in the main Qt loop.
     """
 
-    QT_SLOT_ATTRIBUTE = "__dispatch_qt_slot__"
+    QT_SLOT_ATTRIBUTE: str = "__dispatch_qt_slot__"
+    """Attribute set on functions to be used as slots in the event loop.
+
+    Normally set by by the :meth:`AsyncDispatcher.QtSlot` decorator.
+    """
 
     _callback_executor = _QCallbackExecutor()
 
-    def result(self, timeout: typing.Optional[float] = None) -> _T:  # noqa: UP045
+    def result(self, timeout: float | None = None) -> _T:
         """
         Return the result of the call that the future represents.
 
         Parameters
         ----------
-        timeout: float | None
+        timeout: float | None, optional
             The number of seconds to wait for the result. If None, then wait
             indefinitely.
 
@@ -593,7 +663,7 @@ class DispatcherFuture(Future, typing.Generic[_T]):
 
         Raises
         ------
-        CancelledError
+        asyncio.CancelledError
             If the future was cancelled.
 
         TimeoutError
@@ -604,23 +674,28 @@ class DispatcherFuture(Future, typing.Generic[_T]):
         """  # noqa: DOC502
         return super().result(timeout=timeout)
 
-    def connect(self, fn: typing.Callable[[DispatcherFuture[_T]], None]):
+    def connect(
+        self, fn: collections.abc.Callable[[DispatcherFuture[_T]], None]
+    ) -> None:
         """Attaches a callable that will be called when the future finishes.
 
         The callable will be called by a thread in the same process in which
         it was added if the it was not marked with
-        `DispatherFuture.QT_SLOT_ATTRIBUTE`.
+        :attr:`DispatcherFuture.QT_SLOT_ATTRIBUTE`.
 
-        If the future has already completed or been
-        cancelled then the callable will be called immediately. These
-        callables are called in the order that they were added.
+        If the future has already been cancelld or completed then the callable
+        will be executed immediately. These callables are called in the order
+        that they were added.
 
         Parameters
         ----------
         fn: Callable
-            A callable that will be called with this future's as its only
-            argument when the future completes.
+            A callable that will be executed with this future as its argument
+            once the future completes or is cancelled.
 
+        Returns
+        -------
+        None
         """
         if getattr(fn, self.QT_SLOT_ATTRIBUTE, False):
 
@@ -634,16 +709,17 @@ class DispatcherFuture(Future, typing.Generic[_T]):
 
 
 def run_coroutine_threadsafe(
-    coro: typing.Coroutine[_T, None, _RT], loop: asyncio.AbstractEventLoop,
+    coro: collections.abc.Coroutine[_T, None, _RT],
+    loop: asyncio.AbstractEventLoop,
 ) -> DispatcherFuture[_RT]:
-    """Submit a coroutine object to a given event loop.
+    """Submit a coroutine object to run in the given event loop.
 
     Arguments
     ---------
     coro: Coroutine
         The coroutine object to be submitted.
-    loop: AbstractEventLoop
-        The event loop to run the coroutine.
+    loop: asyncio.AbstractEventLoop
+        The event loop in which to run the coroutine.
 
     Returns
     -------
