@@ -95,6 +95,16 @@ class Debugger(SpyderDockablePlugin, ShellConnectPluginMixin, RunExecutor):
             ]
         }
 
+        self.pyw_editor_run_configuration = {
+            'origin': self.NAME,
+            'extension': 'pyw',
+            'contexts': [
+                {'name': 'File'},
+                {'name': 'Cell'},
+                {'name': 'Selection'},
+            ]
+        }
+
         self.executor_configuration = [
             {
                 'input_extension': 'py',
@@ -112,6 +122,14 @@ class Debugger(SpyderDockablePlugin, ShellConnectPluginMixin, RunExecutor):
                 'requires_cwd': True,
                 'priority': 10
             },
+            {
+                'input_extension': 'pyw',
+                'context': {'name': 'File'},
+                'output_formats': [],
+                'configuration_widget': IPythonConfigOptions,
+                'requires_cwd': True,
+                'priority': 10
+            },
             {
                 'input_extension': 'py',
                 'context': {'name': 'Cell'},
@@ -128,6 +146,14 @@ class Debugger(SpyderDockablePlugin, ShellConnectPluginMixin, RunExecutor):
                 'requires_cwd': True,
                 'priority': 10
             },
+            {
+                'input_extension': 'pyw',
+                'context': {'name': 'Cell'},
+                'output_formats': [],
+                'configuration_widget': None,
+                'requires_cwd': True,
+                'priority': 10
+            },
             {
                 'input_extension': 'py',
                 'context': {'name': 'Selection'},
@@ -144,6 +170,14 @@ class Debugger(SpyderDockablePlugin, ShellConnectPluginMixin, RunExecutor):
                 'requires_cwd': True,
                 'priority': 10
             },
+            {
+                'input_extension': 'pyw',
+                'context': {'name': 'Selection'},
+                'output_formats': [],
+                'configuration_widget': None,
+                'requires_cwd': True,
+                'priority': 10
+            },
         ]
 
     def on_mainwindow_visible(self):
@@ -239,7 +273,8 @@ class Debugger(SpyderDockablePlugin, ShellConnectPluginMixin, RunExecutor):
 
         for run_config in [
             self.python_editor_run_configuration,
-            self.ipython_editor_run_configuration
+            self.ipython_editor_run_configuration,
+            self.pyw_editor_run_configuration
         ]:
             editor.add_supported_run_configuration(run_config)
 
@@ -267,7 +302,8 @@ class Debugger(SpyderDockablePlugin, ShellConnectPluginMixin, RunExecutor):
 
         for run_config in [
             self.python_editor_run_configuration,
-            self.ipython_editor_run_configuration
+            self.ipython_editor_run_configuration,
+            self.pyw_editor_run_configuration
         ]:
             editor.remove_supported_run_configuration(run_config)
 
