@@ -63,6 +63,9 @@ class ScrollFlagArea(Panel):
         # Dictionary with flag lists
         self._dict_flag_list = {}
 
+        # Keep track if todo markers are enabled.
+        self.todo_enabled = True
+
         # Thread to update flags on it.
         self._update_flags_thread = QThread(None)
         self._update_flags_thread.run = self._update_flags
@@ -236,6 +239,11 @@ class ScrollFlagArea(Panel):
         dict_flag_lists_iter = reversed(dict_flag_lists)
 
         for flag_type in dict_flag_lists_iter:
+            # Don't paint todo's if they are not enabled.
+            # Fixes spyder-ide/spyder#24940.
+            if flag_type == 'todo' and not self.todo_enabled:
+                continue
+
             painter.setBrush(self._facecolors[flag_type])
             painter.setPen(self._edgecolors[flag_type])
             if editor.verticalScrollBar().maximum() == 0:
@@ -450,3 +458,7 @@ class ScrollFlagArea(Panel):
         """Toggle scroll flag area visibility"""
         self.enabled = state
         self.setVisible(state)
+
+    def set_todo_enabled(self, state):
+        """Toggle todo flag visibility"""
+        self.todo_enabled = state
