@@ -14,6 +14,8 @@ This is the widget used on all its tabs.
 """
 
 # Standard library imports.
+from __future__ import annotations
+import asyncio
 import functools
 import logging
 import os
@@ -34,6 +36,9 @@ from spyder.api.translations import _
 from spyder.api.widgets.mixins import SpyderWidgetMixin
 from spyder.config.base import (
     get_home_dir, get_module_source_path, get_conf_path)
+from spyder.plugins.remoteclient.api.modules.base import (
+    SpyderRemoteConnectionError,
+)
 from spyder.utils.icon_manager import ima
 from spyder.utils import sourcecode
 from spyder.utils.image_path_manager import get_image_path
@@ -99,6 +104,7 @@ class ClientWidget(QWidget, SaveHistoryMixin, SpyderWidgetMixin):  # noqa: PLR09
     SEPARATOR = '{0}## ---({1})---'.format(os.linesep*2, time.ctime())
     INITHISTORY = ['# -*- coding: utf-8 -*-',
                    '# *** Spyder Python Console History Log ***', ]
+    SHELL_WIDGET_CLASS = ShellWidget
 
     def __init__(
         self,
@@ -154,7 +160,7 @@ class ClientWidget(QWidget, SaveHistoryMixin, SpyderWidgetMixin):  # noqa: PLR09
             self.css_path = css_path
 
         # --- Widgets
-        self.shellwidget = ShellWidget(
+        self.shellwidget = self.SHELL_WIDGET_CLASS(
             config=config_options,
             ipyclient=self,
             additional_options=additional_options,
@@ -166,6 +172,12 @@ class ClientWidget(QWidget, SaveHistoryMixin, SpyderWidgetMixin):  # noqa: PLR09
         self.blank_page = self._create_blank_page()
         self.kernel_loading_page = self._create_loading_page()
         self.env_loading_page = self._create_loading_page(env=True)
+        self.slow_connection_loading_page = self._create_loading_page(
+            message=_("Waiting for the remote kernel to respond...")
+        )
+        self.reconnecting_loading_page = self._create_loading_page(
+            message=_("Reconnecting to remote kernel...")
+        )
 
         if self.is_remote():
             # Keep a reference
@@ -178,6 +190,15 @@ class ClientWidget(QWidget, SaveHistoryMixin, SpyderWidgetMixin):  # noqa: PLR09
         self.t0 = time.monotonic()
         self.timer = QTimer(self)
 
+        # Remote execution timeout feedback
+        self._execution_loading_timeout_ms = 4000
+        self._execution_loading_timer = QTimer(self)
+        self._execution_loading_timer.setSingleShot(True)
+        self._execution_loading_timer.timeout.connect(
+            self._show_slow_execution_message
+        )
+        self._execution_loading_shown = False
+
         # --- Layout
         self.layout = QVBoxLayout()
         self.layout.setContentsMargins(0, 0, 0, 0)
@@ -195,6 +216,8 @@ class ClientWidget(QWidget, SaveHistoryMixin, SpyderWidgetMixin):  # noqa: PLR09
         #--- Remote kernels states
         self.__remote_restart_requested = False
         self.__remote_reconnect_requested = False
+        self._remote_reconnect_attempts_max = 10
+        self._remote_reconnect_attempts = self._remote_reconnect_attempts_max
 
     # ---- Private methods
     # -------------------------------------------------------------------------
@@ -234,16 +257,16 @@ class ClientWidget(QWidget, SaveHistoryMixin, SpyderWidgetMixin):  # noqa: PLR09
         if self.give_focus:
             self.shellwidget._control.setFocus()
 
-    def _create_loading_page(self, env=False):
+    def _create_loading_page(self, env=False, message=None):
         """Create html page to show while the kernel is starting"""
         loading_template = Template(LOADING)
         loading_img = get_image_path('loading_sprites')
         if os.name == 'nt':
             loading_img = loading_img.replace('\\', '/')
-        message = _("Connecting to kernel...")
-
-        if env:
+        if env and message is None:
             message = _("Retrieving environment variables...")
+        if message is None:
+            message = _("Connecting to kernel...")
         page = loading_template.substitute(
             css_path=self.css_path,
             loading_img=loading_img,
@@ -274,6 +297,43 @@ class ClientWidget(QWidget, SaveHistoryMixin, SpyderWidgetMixin):  # noqa: PLR09
             self.set_info_page()
         self.shellwidget.show()
 
+    def _start_execution_loading_timer(self, _=None):
+        """Start timeout to show loading UI for slow remote executions."""
+        if not self.is_remote():
+            return
+
+        self._execution_loading_shown = False
+        self._execution_loading_timer.stop()
+        self._execution_loading_timer.start(
+            self._execution_loading_timeout_ms
+        )
+        logger.debug(
+            "Remote exec loading timer started (%sms)",
+            self._execution_loading_timeout_ms
+        )
+
+    def _show_slow_execution_message(self):
+        """Display loading animation when waiting on a remote execution."""
+        if not self.is_remote():
+            return
+
+        if self.infowidget is not None:
+            self._execution_loading_shown = True
+            self._show_loading_page(self.slow_connection_loading_page)
+        else:
+            self.shellwidget.append_html_message(
+                _("Waiting for the remote kernel to respond..."),
+                before_prompt=True,
+                msg_type='warning',
+            )
+
+    def _stop_execution_loading(self, *_args):
+        """Stop the remote execution loading animation."""
+        self._execution_loading_timer.stop()
+        if self._execution_loading_shown:
+            self._execution_loading_shown = False
+            self._hide_loading_page()
+
     def _show_special_console_error(self, missing_dependency):
         if missing_dependency is not None:
             error_message = _(
@@ -372,6 +432,34 @@ class ClientWidget(QWidget, SaveHistoryMixin, SpyderWidgetMixin):  # noqa: PLR09
                     cwd_path, emit_cwd_change=emit_cwd_change
                 )
 
+    def _render_error_page(self, error: str | Exception, message: str):
+        if isinstance(error, Exception):
+            if isinstance(error, SpyderKernelError):
+                error_text = error.args[0]
+            else:
+                error_text = _("The error is:<br><br><tt>{}</tt>").format(
+                    traceback.format_exc()
+                )
+        else:
+            error_text = error
+
+        # Replace end of line chars with <br>
+        eol = sourcecode.get_eol_chars(error_text)
+        if eol:
+            error_text = error_text.replace(eol, '<br>')
+
+        # Don't break lines in hyphens
+        # From https://stackoverflow.com/q/7691569/438386
+        error_text = error_text.replace('-', '&#8209;')
+
+        # Create error page
+        kernel_error_template = Template(KERNEL_ERROR)
+        page =  kernel_error_template.substitute(
+            css_path=self.css_path, message=message, error=error_text
+        )
+
+        return page, error_text
+
     # ---- Public API
     # -------------------------------------------------------------------------
     @property
@@ -506,38 +594,21 @@ class ClientWidget(QWidget, SaveHistoryMixin, SpyderWidgetMixin):  # noqa: PLR09
 
     def show_kernel_error(self, error):
         """Show kernel initialization errors in infowidget."""
-        if isinstance(error, Exception):
-            if isinstance(error, SpyderKernelError):
-                error = error.args[0]
-            else:
-                error = _("The error is:<br><br>"
-                          "<tt>{}</tt>").format(traceback.format_exc())
-        self.error_text = error
-
-        if self.is_benign_error(error):
-            return
-
-        if self.is_warning_message(error):
-            return
-
-        InstallerIPythonKernelError(error)
-
-        # Replace end of line chars with <br>
-        eol = sourcecode.get_eol_chars(error)
-        if eol:
-            error = error.replace(eol, '<br>')
-
-        # Don't break lines in hyphens
-        # From https://stackoverflow.com/q/7691569/438386
-        error = error.replace('-', '&#8209;')
-
         # Create error page
         message = _("An error occurred while starting the kernel")
-        kernel_error_template = Template(KERNEL_ERROR)
-        self.info_page = kernel_error_template.substitute(
-            css_path=self.css_path,
-            message=message,
-            error=error)
+        self.info_page, error_text = self._render_error_page(
+            error, message
+        )
+
+        self.error_text = error_text
+
+        if self.is_benign_error(error_text):
+            return
+
+        if self.is_warning_message(error_text):
+            return
+
+        InstallerIPythonKernelError(error_text)
 
         # Show error
         if self.infowidget is not None:
@@ -691,6 +762,9 @@ class ClientWidget(QWidget, SaveHistoryMixin, SpyderWidgetMixin):  # noqa: PLR09
         except (RuntimeError, TypeError):
             pass
 
+        if self.is_remote():
+            self._execution_loading_timer.stop()
+
         # This is a hack to prevent segfaults when closing Spyder and the
         # client was debugging before doing it.
         # It's a side effect of spyder-ide/spyder#21788
@@ -743,7 +817,13 @@ class ClientWidget(QWidget, SaveHistoryMixin, SpyderWidgetMixin):  # noqa: PLR09
         except RuntimeError:
             pass
 
-    def replace_kernel(self, kernel_handler, shutdown_kernel, clear=True):
+    def replace_kernel(
+        self,
+        kernel_handler,
+        shutdown_kernel,
+        clear=True,
+        reconnect=False,
+    ):
         """
         Replace kernel by disconnecting from the current one and connecting to
         another kernel, which is equivalent to a restart.
@@ -757,7 +837,10 @@ class ClientWidget(QWidget, SaveHistoryMixin, SpyderWidgetMixin):  # noqa: PLR09
 
         # Reset shellwidget and print restart message
         self.shellwidget.reset(clear=clear)
-        self.shellwidget._kernel_restarted_message(died=False)
+        self.shellwidget._kernel_restarted_message(
+            died=False,
+            reconnected=reconnect,
+        )
 
     def is_kernel_active(self):
         """Check if the kernel is active."""
@@ -873,90 +956,137 @@ class ClientWidget(QWidget, SaveHistoryMixin, SpyderWidgetMixin):  # noqa: PLR09
     def jupyter_api(self):
         return self._jupyter_api
 
-    def remote_kernel_restarted_failure_message(
-        self, error=None, shutdown=False
+    def _show_remote_kernel_error(self, message, error: str | Exception):
+        self.info_page, error_text = self._render_error_page(error, message)
+
+        # Prefer showing in the info widget to avoid mixing with console I/O
+        if self.infowidget is not None:
+            self.set_info_page()
+            self.shellwidget.hide()
+            self.infowidget.show()
+        else:
+            # Fallback inline if info widget is unavailable
+            self.shellwidget._append_html(
+                f"<br>{message}<br>", before_prompt=False
+            )
+            self.shellwidget.insert_horizontal_ruler()
+
+        # Inform other plugins that the shell failed to reconnect
+        self.shellwidget.sig_shellwidget_errored.emit(self.shellwidget)
+
+    def remote_kernel_restarted_failure(
+        self, error: str | Exception, shutdown: bool = False
     ):
         """Show message when the kernel failed to be restarted."""
-
-        msg = _("It was not possible to restart the kernel")
-
-        if error is None:
-            error_html = f"<br>{msg}<br>"
-        else:
-            if isinstance(error, SpyderKernelError):
-                error = error.args[0]
-            elif isinstance(error, Exception):
-                error = _("The error is:<br><br>" "<tt>{}</tt>").format(
-                    traceback.format_exc()
-                )
-
-            # Replace end of line chars with <br>
-            eol = sourcecode.get_eol_chars(error)
-            if eol:
-                error = error.replace(eol, '<br>')
-
-            # Don't break lines in hyphens
-            # From https://stackoverflow.com/q/7691569/438386
-            error = error.replace('-', '&#8209')
-
-            # Create error page
-            kernel_error_template = Template(KERNEL_ERROR)
-            error_html = kernel_error_template.substitute(
-                css_path=self.css_path,
-                message=msg,
-                error=error)
-
-        self.shellwidget._append_html(error_html, before_prompt=False)
-        self.shellwidget.insert_horizontal_ruler()
+        msg = _("An error occurred while restarting the remote kernel")
+        self._show_remote_kernel_error(msg, error)
 
         if shutdown:
             self.shutdown(is_last_client=False, close_console=False)
 
+    def remote_kernel_reconnect_failure(self, error: str | Exception):
+        """Handle failures while trying to reconnect to a remote kernel."""
+        msg = _("An error occurred while reconnecting to the remote kernel")
+        self._show_remote_kernel_error(msg, error)
+
     @AsyncDispatcher.QtSlot
     def _on_remote_kernel_restarted(self, future):
         """Handle restarts for remote kernels."""
-        if future.result():
-            # Reset shellwidget and print restart message
-            self.kernel_handler.reconnect_kernel()
-            self.shellwidget.reset(clear=True)
+        try:
+            restarted = future.result()
+        except Exception as err:
+            logger.debug(
+                "Failed to restart remote kernel",
+                exc_info=True,
+            )
+            self.remote_kernel_restarted_failure(error=err, shutdown=True)
         else:
-            self.remote_kernel_restarted_failure_message(shutdown=True)
-            # This will show an error message in the plugins connected to the
-            # IPython console and disable kernel related actions in its Options
-            # menu.
-            sw = self.shellwidget
-            sw.sig_shellwidget_errored.emit(sw)
-        self.__remote_restart_requested = False
+            if restarted:
+                self.kernel_handler.reconnect_kernel()
+            else:
+                self.remote_kernel_restarted_failure(
+                    error=_(
+                        "It was not possible to restart the remote kernel"
+                    ),
+                    shutdown=True,
+                )
+        finally:
+            self.__remote_restart_requested = False
 
     @AsyncDispatcher.QtSlot
     def _reconnect_on_kernel_info(self, future):
-        if (kernel_info := future.result()):
-            try:
-                kernel_handler = KernelHandler.from_websocket(
-                    (
-                        self.jupyter_api.api_url
-                        / "kernels"
-                        / self.kernel_id
-                        / "channels"
-                    ).with_scheme("ws"),
-                    aiohttp_session=self._jupyter_api.session,
+        try:
+            kernel_info = future.result()
+        except SpyderRemoteConnectionError:
+            if self._remote_reconnect_attempts <= 0:
+                logger.debug(
+                    "Exceeded max attempts to reconnect to remote kernel",
+                    exc_info=True,
                 )
-            except Exception as err:
-                self.remote_kernel_restarted_failure_message(
-                    err, shutdown=True
-                )
-            else:
-                self.replace_kernel(
-                    kernel_handler, shutdown_kernel=False, clear=False
+                self.remote_kernel_reconnect_failure(
+                    _(
+                        "The maximum number of attempts to reconnect to the "
+                        "kernel was exceeded."
+                    )
                 )
+                self.__remote_reconnect_requested = False
+                return
+            self._remote_reconnect_attempts -= 1
+            logger.debug(
+                "Reconnecting to remote kernel, attempts left: %d",
+                self._remote_reconnect_attempts,
+            )
+            self.__remote_reconnect_requested = False
+            self.reconnect_remote_kernel(delay=5)
+            return
+        except Exception as err:
+            logger.debug(
+                "Unexpected error while getting remote kernel info",
+                exc_info=True,
+            )
+            self.remote_kernel_reconnect_failure(error=err)
+            self.__remote_reconnect_requested = False
+            return
         else:
-            self.remote_kernel_restarted_failure_message(shutdown=True)
-            # This will show an error message in the plugins connected to the
-            # IPython console and disable kernel related actions in its Options
-            # menu.
-            sw = self.shellwidget
-            sw.sig_shellwidget_errored.emit(sw)
-        self.__remote_reconnect_requested = False
+            if not kernel_info:
+                self.remote_kernel_reconnect_failure(
+                    _(
+                        "The remote kernel info was not found. Most probably "
+                        "the server was restarted."
+                    )
+                )
+                self.__remote_reconnect_requested = False
+                return
+
+        try:
+            kernel_handler = KernelHandler.from_websocket(
+                (
+                    self.jupyter_api.api_url
+                    / "kernels"
+                    / self.kernel_id
+                    / "channels"
+                ).with_scheme("ws"),
+                aiohttp_session=self._jupyter_api.session,
+            )
+        except Exception as err:
+            logger.debug(
+                "Failed to create KernelHandler for remote kernel",
+                exc_info=True,
+            )
+            self.remote_kernel_reconnect_failure(error=err)
+        else:
+            self._remote_reconnect_attempts = (
+                self._remote_reconnect_attempts_max
+            )
+            self._hide_loading_page()
+            self.replace_kernel(
+                kernel_handler,
+                shutdown_kernel=False,
+                clear=False,
+                reconnect=True,
+            )
+        finally:
+            self.__remote_reconnect_requested = False
 
     @AsyncDispatcher.QtSlot
     def _on_remote_kernel_started(self, future):
@@ -967,21 +1097,23 @@ class ClientWidget(QWidget, SaveHistoryMixin, SpyderWidgetMixin):  # noqa: PLR09
         # It's only at this point that we can allow users to close the client.
         self.can_close = True
 
-        # Handle failures to launch a kernel
+        # The future returns the kernel info or an error.
         try:
             kernel_info = future.result()
-        except Exception as err:
-            self.show_kernel_error(err)
-            return
-
-        if not kernel_info:
+        except SpyderRemoteConnectionError:
             self.show_kernel_error(
                 _(
-                    "There was an error connecting to the server <b>{}</b>. "
-                    "Please check your connection is working."
-                ).format(self._jupyter_api.server_name)
+                    "There was an error connecting to the server <b>{}</b>."
+                    "<br><br>"
+                    "Please check your connection is working in the menu "
+                    "entry <tt>Tools > Manage remote connections</tt>."
+                ).format(self.jupyter_api.server_name)
             )
             return
+        except Exception as err:
+            logger.debug("Failed to start remote kernel", exc_info=True)
+            self.show_kernel_error(error=err)
+            return
 
         # Connect client's signals
         self.kernel_id = kernel_info["id"]
@@ -997,7 +1129,11 @@ class ClientWidget(QWidget, SaveHistoryMixin, SpyderWidgetMixin):  # noqa: PLR09
                 aiohttp_session=self._jupyter_api.session,
             )
         except Exception as err:
-            self.show_kernel_error(err)
+            logger.debug(
+                "Failed to create KernelHandler for remote kernel",
+                exc_info=True,
+            )
+            self.show_kernel_error(error=err)
         else:
             # Connect client to the kernel
             self.connect_kernel(kernel_handler)
@@ -1015,7 +1151,12 @@ class ClientWidget(QWidget, SaveHistoryMixin, SpyderWidgetMixin):  # noqa: PLR09
         return await self._jupyter_api.restart_kernel(self.kernel_id)
 
     @AsyncDispatcher(loop="ipythonconsole")
-    async def _get_remote_kernel_info(self):
+    async def _get_remote_kernel_info(self, delay=0):
+        if delay > 0:
+            await asyncio.sleep(delay)
+        # Reensure connection before getting kernel info
+        await self.jupyter_api.close()
+        await self.jupyter_api.connect()
         return await self._jupyter_api.get_kernel(self.kernel_id)
 
     @AsyncDispatcher(loop="ipythonconsole")
@@ -1047,10 +1188,14 @@ class ClientWidget(QWidget, SaveHistoryMixin, SpyderWidgetMixin):  # noqa: PLR09
         )
         self.__remote_restart_requested = True
 
-    def reconnect_remote_kernel(self):
+    def reconnect_remote_kernel(self, delay=0):
         if self.__remote_reconnect_requested:
             return
-        self._get_remote_kernel_info().connect(self._reconnect_on_kernel_info)
+        if self.infowidget is not None:
+            self._show_loading_page(self.reconnecting_loading_page)
+        self._get_remote_kernel_info(delay=delay).connect(
+            self._reconnect_on_kernel_info
+        )
         self.__remote_reconnect_requested = True
 
     def start_remote_kernel(self, kernel_spec=None):
