@@ -271,9 +271,11 @@ class RunContainer(PluginMainContainer):
 
         return anonymous_execution_run
 
-    def run_file(self, selected_uuid=None, selected_executor=None):
+    def run_file(
+        self, selected_uuid=None, selected_executor=None, rerun=False
+    ):
         if not isinstance(selected_uuid, bool) and selected_uuid is not None:
-            self.switch_focused_run_configuration(selected_uuid)
+            self.switch_focused_run_configuration(selected_uuid, rerun)
 
         self.edit_run_configurations(
             display_dialog=False,
@@ -384,13 +386,17 @@ class RunContainer(PluginMainContainer):
             executor.exec_run_configuration(run_conf, ext_params)
 
     def re_run_file(self):
-        self.run_file(self.last_executed_file, selected_executor=None)
+        self.run_file(
+            self.last_executed_file, selected_executor=None, rerun=True
+        )
 
     @property
     def currently_selected_configuration(self):
         return self.metadata_model.get_current_run_configuration()
 
-    def switch_focused_run_configuration(self, uuid: Optional[str]):
+    def switch_focused_run_configuration(
+        self, uuid: Optional[str], rerun: bool = False
+    ):
         uuid = uuid or None
 
         # We need the first check to correctly update the run and context
@@ -409,8 +415,10 @@ class RunContainer(PluginMainContainer):
             self.current_input_provider = metadata['source']
             self.current_input_extension = metadata['input_extension']
 
-            input_provider = self.run_metadata_provider[uuid]
-            input_provider.focus_run_configuration(uuid)
+            if not rerun:
+                input_provider = self.run_metadata_provider[uuid]
+                input_provider.focus_run_configuration(uuid)
+
             self.set_actions_status()
 
             return
