@@ -25,7 +25,7 @@ from qtpy.QtWidgets import (QAction, QGridLayout, QHBoxLayout, QLabel,
 
 # Local imports
 from spyder.api.shortcuts import SpyderShortcutsMixin
-from spyder.config.base import _
+from spyder.api.translations import _
 from spyder.utils.icon_manager import ima
 from spyder.utils.misc import regexp_error_msg
 from spyder.plugins.editor.utils.editor import TextHelper
@@ -307,7 +307,7 @@ class FindReplace(QWidget, SpyderShortcutsMixin):
             key = event.key()
             shift = event.modifiers() & Qt.ShiftModifier
 
-            if key == Qt.Key_Return:
+            if key in (Qt.Key_Return, Qt.Key_Enter):
                 if shift:
                     self.return_shift_pressed.emit()
                 else:
